#
# spec file for package python-Cython
#
# Copyright (c) 2015 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


Name:           python-Cython
Version:        0.22
Release:        1.6
Url:            http://www.cython.org
Summary:        The Cython compiler for writing C extensions for the Python language
License:        Apache-2.0
Group:          Development/Languages/Python
Source:         http://cython.org/release/Cython-%{version}.tar.gz
Source1:        python-Cython-rpmlintrc
# PATCH-FIX-UPSTREAM -- https://github.com/cython/cython/commit/65408e6c70074c4fec128805888d18c7e933895a
Patch0:         fix-32bit.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildRequires:  fdupes
BuildRequires:  gcc-c++
BuildRequires:  python-devel
BuildRequires:  python-numpy
BuildRequires:  python-numpy-devel
BuildRequires:  python-xml
Provides:       python-cython = %{version}
Obsoletes:      python-cython < %{version}
Requires:       python-xml
Requires(post): update-alternatives
Requires(postun): update-alternatives
%if 0%{?suse_version} && 0%{?suse_version} <= 1110
%{!?python_sitearch: %global python_sitearch %(python -c "from distutils.sysconfig import get_python_lib; print get_python_lib(1)")}
%endif

%description
The Cython language makes writing C extensions for the Python language as
easy as Python itself.  Cython is a source code translator based on the
well-known Pyrex, but supports more cutting edge functionality and
optimizations.

The Cython language is very close to the Python language (and most Python
code is also valid Cython code), but Cython additionally supports calling C
functions and declaring C types on variables and class attributes. This
allows the compiler to generate very efficient C code from Cython code.

This makes Cython the ideal language for writing glue code for external C
libraries, and for fast C modules that speed up the execution of Python
code.

%prep
%setup -q -n Cython-%{version}
# Fix non-executable scripts
sed -i "s|^#!.*||" Cython/Debugger/{libpython,Cygdb}.py cython.py
# Fix EOL encoding
sed -i "s|\r||" Demos/callback/{README.txt,cheesefinder.h} Demos/embed/Makefile.{unix,msc.static} Doc/primes.c
%patch0 -p1

%build
CFLAGS="%{optflags}" python setup.py build

%install
python setup.py install --prefix=%{_prefix} --root=%{buildroot}
# Prepare for update-alternatives usage
mkdir -p %{buildroot}%{_sysconfdir}/alternatives
for p in cython cythonize cygdb ; do
    mv %{buildroot}%{_bindir}/$p %{buildroot}%{_bindir}/$p-%{py_ver}
    ln -s %{_bindir}/$p-%{py_ver} %{buildroot}%{_bindir}/$p
    ln -s %{_bindir}/$p-%{py_ver} %{buildroot}%{_sysconfdir}/alternatives/$p
done

%fdupes -s %{buildroot}%{python_sitearch} %{buildroot}%{_docdir}
rm -rf %{buildroot}%{python_sitearch}/__pycache__/
# avoid conflicts with python 2 version

%check
python runtests.py -vv

%post
%_sbindir/update-alternatives \
   --install %{_bindir}/cython cython %{_bindir}/cython-%{py_ver} 30 \
   --slave %{_bindir}/cythonize cythonize %{_bindir}/cythonize-%{py_ver} \
   --slave %{_bindir}/cygdb cygdb %{_bindir}/cygdb-%{py_ver}

%preun
if [ $1 -eq 0 ] ; then
    %_sbindir/update-alternatives --remove cython %{_bindir}/cython-%{py_ver}
fi

%files
%defattr(-,root,root,-)
%doc COPYING.txt LICENSE.txt README.txt ToDo.txt USAGE.txt Doc Demos
%ghost %{_bindir}/cygdb
%ghost %{_bindir}/cython
%ghost %{_bindir}/cythonize
%{_bindir}/cygdb-%{py_ver}
%{_bindir}/cython-%{py_ver}
%{_bindir}/cythonize-%{py_ver}
%ghost %{_sysconfdir}/alternatives/cygdb
%ghost %{_sysconfdir}/alternatives/cython
%ghost %{_sysconfdir}/alternatives/cythonize
%{python_sitearch}/Cython/
%{python_sitearch}/Cython-%{version}-py%{py_ver}.egg-info
%{python_sitearch}/cython.py*
%{python_sitearch}/pyximport/

%changelog
* Wed Apr 22 2015 mcihar@suse.cz
- Use Source URL from cython.org
* Wed Apr 22 2015 mcihar@suse.cz
- Add python-numpy as BuildRequires to have more complete test coverage
* Wed Apr 22 2015 mcihar@suse.cz
- Fix doctests in 32-bit environment (fix-32bit.patch)
* Wed Apr 22 2015 mcihar@suse.cz
- Update to 0.22:
  Features added
  * C functions can coerce to Python functions, which allows passing them
    around as callable objects.
  * C arrays can be assigned by value and auto-coerce from Python iterables
    and to Python lists (and tuples).
  * Extern C functions can now be declared as cpdef to export them to
    the module's Python namespace.  Extern C functions in pxd files export
    their values to their own module, iff it exists.
  * Anonymous C tuple types can be declared as (ctype1, ctype2, ...).
  * PEP 479: turn accidental StopIteration exceptions that exit generators
    into a RuntimeError, activated with future import "generator_stop".
    See http://legacy.python.org/dev/peps/pep-0479/
  * Looping over ``reversed(range())`` is optimised in the same way as
    ``range()``.  Patch by Favian Contreras.
  Bugs fixed
  * Mismatching 'except' declarations on signatures in .pxd and .pyx files failed
    to produce a compile error.
  * Failure to find any files for the path pattern(s) passed into ``cythonize()``
    is now an error to more easily detect accidental typos.
  * The ``logaddexp`` family of functions in ``numpy.math`` now has correct
    declarations.
  * In Py2.6/7 and Py3.2, simple Cython memory views could accidentally be
    interpreted as non-contiguous by CPython, which could trigger a CPython
    bug when copying data from them, thus leading to data corruption.
    See CPython issues 12834 and 23349.
  Other changes
  * Preliminary support for defining the Cython language with a formal grammar.
    To try parsing your files against this grammar, use the --formal_grammar directive.
    Experimental.
  * ``_`` is no longer considered a cacheable builtin as it could interfere with
    gettext.
  * Cythonize-computed metadata now cached in the generated C files.
* Thu Feb  5 2015 hpj@urpla.net
- fix update-alternatives handling in a distribution backwards compatible way
* Fri Jan  9 2015 dimstar@opensuse.org
- Re-enable test-suite.
- Add gcc-c++ BuildRequires: needed for the test-suite to be able
  to pass.
* Thu Jan  8 2015 dimstar@opensuse.org
- Fix usage of update-alternatives.
* Thu Dec 18 2014 p.drouand@gmail.com
- Improve update-alternatives.
- Remove Cython-fix-version-detection.patch
  (got fixed upstream)
- update to version 0.21.1:
  * Features added
  - New cythonize option -a to generate the annotated HTML source view.
  - Missing C-API declarations in cpython.unicode were added.
  - Passing language='c++' into cythonize() globally enables C++ mode
    for all modules that were not passed as Extension objects
    (i.e. only source files and file patterns).
  - Py_hash_t is a known type (used in CPython for hash values).
  - PySlice_*() C-API functions are available from the cpython.slice module.
  - Allow arrays of C++ classes.
  * Bugs fixed
  - Reference leak for non-simple Python expressions in boolean and/or expressions.
  - To fix a name collision and to reflect availability on host
    platforms, standard C declarations [ clock(), time(), struct tm
    and tm* functions ] were moved from posix/time.pxd to a new
    libc/time.pxd. Patch by Charles Blake.
  - Rerunning unmodified modules in IPython's cython support
    failed. Patch by Matthias Bussonier.
  - Casting C++ std::string to Python byte strings failed when
    auto-decoding was enabled.
  - Fatal exceptions in global module init code could lead to
    crashes if the already created module was used later on
    (e.g. through a stale reference in sys.modules or elsewhere).
  - cythonize.py script was not installed on MS-Windows.
  * Other changes
  - Compilation no longer fails hard when unknown compilation
    options are passed. Instead, it raises a warning and ignores
    them (as it did silently before 0.21). This will be changed back
    to an error in a future release.
* Sun Nov  9 2014 ledest@gmail.com
- fix bashisms in pre script
* Fri Sep 12 2014 toddrme2178@gmail.com
- Add Cython-fix-version-detection.patch
  This is a patch from upstream that restores version information
  whose removal is preventing several packages from correctly
  detecting Cython's presence.  It is already merged upstream and
  so should be in the next release.
  Note that despite what upstream says,
  python-tables/python3-tables is NOT the only package affected by
  this, which is why the patch is going here instead of
  python-tables/python3-tables.
  python-bcolz/python3-bcolz is an example of another package
  affected.
* Thu Sep 11 2014 toddrme2178@gmail.com
- Update to 0.21 (2014-09-10)
  * Features added
  * C (cdef) functions allow inner Python functions.
  * Enums can now be declared as cpdef to export their values to
    the module's Python namespace.  Cpdef enums in pxd files export
    their values to their own module, iff it exists.
  * Allow @staticmethod decorator to declare static cdef methods.
    This is especially useful for declaring "constructors" for
    cdef classes that can take non-Python arguments.
  * Taking a ``char*`` from a temporary Python string object is safer
    in more cases and can be done inside of non-trivial expressions,
    including arguments of a function call.  A compile time error
    is raised only when such a pointer is assigned to a variable and
    would thus exceed the lifetime of the string itself.
  * Generators have new properties ``__name__`` and ``__qualname__``
    that provide the plain/qualified name of the generator function
    (following CPython 3.5).  See http://bugs.python.org/issue21205
  * The ``inline`` function modifier is available as a decorator
    ``@cython.inline`` in pure mode.
  * When cygdb is run in a virtualenv, it enables the same virtualenv
    inside of the debugger. Patch by Marc Abramowitz.
  * PEP 465: dedicated infix operator for matrix multiplication (A @ B).
  * HTML output of annotated code uses Pygments for code highlighting
    and generally received a major overhaul by Matthias Bussonier.
  * IPython magic support is now available directly from Cython with
    the command "%%load_ext cython".  Cython code can directly be
    executed in a cell when marked with "%%%%cython".  Code analysis
    is available with "%%%%cython -a".  Patch by Martín Gaitán.
  * Simple support for declaring Python object types in Python signature
    annotations.  Currently requires setting the compiler directive
    ``annotation_typing=True``.
  * New directive ``use_switch`` (defaults to True) to optionally disable
    the optimization of chained if statement to C switch statements.
  * Defines dynamic_cast et al. in ``libcpp.cast`` and C++ heap data
    structure operations in ``libcpp.algorithm``.
  * Shipped header declarations in ``posix.*`` were extended to cover
    more of the POSIX API.  Patches by Lars Buitinck and Mark Peek.
  * Optimizations
  * Simple calls to C implemented Python functions/methods are faster.
    This also speeds up many operations on builtins that Cython cannot
    otherwise optimise.
  * The "and"/"or" operators try to avoid unnecessary coercions of their
    arguments.  They now evaluate the truth value of each argument
    independently and only coerce the final result of the whole expression
    to the target type (e.g. the type on the left side of an assignment).
    This also avoids reference counting overhead for Python values during
    evaluation and generally improves the code flow in the generated C code.
  * The Python expression "2 ** N" is optimised into bit shifting.
    See http://bugs.python.org/issue21420
  * Cascaded assignments (a = b = ...) try to minimise the number of
    type coercions.
  * Calls to ``slice()`` are translated to a straight C-API call.
  * Bugs fixed
  * Crash when assigning memory views from ternary conditional expressions.
  * Nested C++ templates could lead to unseparated ">>" characters being
    generated into the C++ declarations, which older C++ compilers could
    not parse.
  * Sending SIGINT (Ctrl-C) during parallel cythonize() builds could
    hang the child processes.
  * No longer ignore local setup.cfg files for distutils in pyximport.
    Patch by Martin Teichmann.
  * Taking a ``char*`` from an indexed Python string generated unsafe
    reference counting code.
  * Set literals now create all of their items before trying to add them
    to the set, following the behaviour in CPython.  This makes a
    difference in the rare case that the item creation has side effects
    and some items are not hashable (or if hashing them has side effects,
    too).
  * Cython no longer generates the cross product of C functions for code
    that uses memory views of fused types in function signatures (e.g.
    ``cdef func(floating[:] a, floating[:] b)``).  This is considered the
    expected behaviour by most users and was previously inconsistent with
    other structured types like C arrays.  Code that really wants all type
    combinations can create the same fused memoryview type under different
    names and use those in the signature to make it clear which types are
    independent.
  * Names that were unknown at compile time were looked up as builtins at
    runtime but not as global module names.  Trying both lookups helps with
    globals() manipulation.
  * Fixed stl container conversion for typedef element types.
  * ``obj.pop(x)`` truncated large C integer values of x to ``Py_ssize_t``.
  * ``__init__.pyc`` is recognised as marking a package directory
    (in addition to .py, .pyx and .pxd).
  * Syntax highlighting in ``cython-mode.el`` for Emacs no longer
    incorrectly highlights keywords found as part of longer names.
  * Correctly handle ``from cython.submodule cimport name``.
  * Fix infinite recursion when using super with cpdef methods.
  * No-args ``dir()`` was not guaranteed to return a sorted list.
  * Other changes
  * The header line in the generated C files no longer contains the
    timestamp but only the Cython version that wrote it.  This was
    changed to make builds more reproducible.
  * Removed support for CPython 2.4, 2.5 and 3.1.
  * The licensing implications on the generated code were clarified
    to avoid legal constraints for users.
* Thu Jul 31 2014 dimstar@opensuse.org
- Rename rpmlintrc to %%{name}-rpmlintrc.
  Follow the packaging guidelines.
* Thu Jul 24 2014 mcihar@suse.cz
- Update to version 0.20.2:
  * Some optimisations for set/frozenset instantiation.
  * Support for C++ unordered_set and unordered_map.
  * Access to attributes of optimised builtin methods (e.g.
    ``[].append.__name__``) could fail to compile.
  * Memory leak when extension subtypes add a memory view as attribute
    to those of the parent type without having Python object attributes
    or a user provided dealloc method.
  * Compiler crash on readonly properties in "binding" mode.
  * Auto-encoding with ``c_string_encoding=ascii`` failed in Py3.3.
  * Crash when subtyping freelist enabled Cython extension types with
    Python classes that use ``__slots__``.
  * Freelist usage is restricted to CPython to avoid problems with other
    Python implementations.
  * Memory leak in memory views when copying overlapping, contiguous slices.
  * Format checking when requesting non-contiguous buffers from
    ``cython.array`` objects was disabled in Py3.
  * C++ destructor calls in extension types could fail to compile in clang.
  * Buffer format validation failed for sequences of strings in structs.
  * Docstrings on extension type attributes in .pxd files were rejected.
- add python-xml to build requirements for testsuite
* Thu May  8 2014 toddrme2178@gmail.com
- Update to version 0.20.1
  * List/Tuple literals multiplied by more than one factor were only multiplied
    by the last factor instead of all.
  * Lookups of special methods (specifically for context managers) could fail
    in Python <= 2.6/3.1.
  * Local variables were erroneously appended to the signature introspection
    of Cython implemented functions with keyword-only arguments under Python 3.
  * In-place assignments to variables with inferred Python builtin/extension
    types could fail with type errors if the result value type was incompatible
    with the type of the previous value.
  * The C code generation order of cdef classes, closures, helper code,
    etc. was not deterministic, thus leading to high code churn.
  * Type inference could fail to deduce C enum types.
  * Type inference could deduce unsafe or inefficient types from integer
    assignments within a mix of inferred Python variables and integer
    variables.
* Mon Feb  3 2014 jengelh@inai.de
- Cython grew a dependency on saxutils (and since then, libplist
  failed to build). Add python-xml as a Requires to avoid:
  [...]
  File "/usr/lib64/python2.7/site-packages/Cython/Compiler/Annotate.py",
  line 6, in <module> from xml.sax.saxutils import escape as
  html_escape.
  ImportError: No module named xml.sax.saxutils
* Fri Jan 31 2014 speilicke@suse.com
- Update to version 0.20:
  * Support for CPython 3.4.
  * Support for calling C++ template functions.
  * yield is supported in finally clauses.
  * The C code generated for finally blocks is duplicated for each exit
    case to allow for better optimisations by the C compiler.
  * Cython tries to undo the Python optimisationism of assigning a bound
    method to a local variable when it can generate better code for the
    direct call.
  * Constant Python float values are cached.
  * String equality comparisons can use faster type specific code in
    more cases than before.
  * String/Unicode formatting using the '%%' operator uses a faster
    C-API call.
  * bytearray has become a known type and supports coercion from and
    to C strings.  Indexing, slicing and decoding is optimised. Note that
    this may have an impact on existing code due to type inference.
  * Using cdef basestring stringvar and function arguments typed as
    basestring is now meaningful and allows assigning exactly
    str and unicode objects, but no subtypes of these types.
  * Support for the __debug__ builtin.
  * Assertions in Cython compiled modules are disabled if the running
    Python interpreter was started with the "-O" option.
  * Some types that Cython provides internally, such as functions and
    generators, are now shared across modules if more than one Cython
    implemented module is imported.
  * The type inference algorithm works more fine granular by taking the
    results of the control flow analysis into account.
  * A new script in bin/cythonize provides a command line frontend
    to the cythonize() compilation function (including distutils build).
  * The new extension type decorator @cython.no_gc_clear prevents
    objects from being cleared during cyclic garbage collection, thus
    making sure that object attributes are kept alive until deallocation.
  * During cyclic garbage collection, attributes of extension types that
    cannot create reference cycles due to their type (e.g. strings) are
    no longer considered for traversal or clearing.  This can reduce the
    processing overhead when searching for or cleaning up reference cycles.
  * Package compilation (i.e. __init__.py files) now works, starting
    with Python 3.3.
  * The cython-mode.el script for Emacs was updated.  Patch by Ivan Andrus.
  * An option common_utility_include_dir was added to cythonize() to save
    oft-used utility code once in a separate directory rather than as
    part of each generated file.
  * unraisable_tracebacks directive added to control printing of
    tracebacks of unraisable exceptions.
* Mon Oct 21 2013 dmueller@suse.com
- update to 0.19.2:
  * Some standard declarations were fixed or updated, including the previously
    incorrect declaration of PyBuffer_FillInfo() and some missing bits in
    libc.math.
  * Heap allocated subtypes of type used the wrong base type struct at the
    C level.
  * Calling the unbound method dict.keys/value/items() in dict subtypes could
    call the bound object method instead of the unbound supertype method.
  * "yield" wasn't supported in "return" value expressions.
  * Using the "bint" type in memory views lead to unexpected results.
    It is now an error.
  * Assignments to global/closure variables could catch them in an illegal state
    while deallocating the old value.
* Fri Oct 11 2013 p.drouand@gmail.com
- Implement update-alternatives
* Mon May 13 2013 dmueller@suse.com
- update to 0.19.1:
  * Completely empty C-API structs for extension type slots (protocols like
  number/mapping/sequence) are no longer generated into the C code.
  * Docstrings that directly follow a public/readonly attribute declaration
  in a cdef class will be used as docstring of the auto-generated property.
  This fixes ticket 206.
  * The automatic signature documentation tries to preserve more semantics
  of default arguments and argument types.  Specifically, bint arguments
  now appear as type bool.
  * A warning is emitted when negative literal indices are found inside of
  a code section that disables wraparound handling.  This helps with
  fixing invalid code that might fail in the face of future compiler
  optimisations.
  * Constant folding for boolean expressions (and/or) was improved.
  * Added a build_dir option to cythonize() which allows one to place
  the generated .c files outside the source tree.
* Mon Apr 29 2013 dmueller@suse.com
- Update to version 0.19:
  + Please see http://wiki.cython.org/ReleaseNotes-0.19
* Wed Mar 27 2013 speilicke@suse.com
- Update to version 0.18:
  + Please see http://wiki.cython.org/ReleaseNotes-0.18
* Thu Nov 22 2012 saschpe@suse.de
- Update to version 0.17.2:
  + Please see http://wiki.cython.org/ReleaseNotes-0.17.2
- Drop excessive macro usage
- No need for "-fno-strict-aliasing" anymore
- One rpmlintrc is enough
* Fri May 25 2012 toddrme2178@gmail.com
- Fix .py/.pyc issues
* Fri May 18 2012 toddrme2178@gmail.com
- Add python 3 package
- Clean up spec file formatting
- Remove setbadness from rplintrc files
* Mon Apr 23 2012 vdziewiecki@suse.com
-Update to 0.16:
  http://wiki.cython.org/ReleaseNotes-0.16
* Wed Feb 29 2012 pascal.bleser@opensuse.org
- add rpmlintrc to mask false positives
* Wed Sep 21 2011 saschpe@suse.de
- Update to version 0.15.1:
  * Please see http://wiki.cython.org/ReleaseNotes-0.15.1
* Tue Sep  6 2011 saschpe@suse.de
- Add Provides/Obsoletes for python-cython
* Fri Sep  2 2011 saschpe@suse.de
- Fixed a typo
- Removed testsuite again, fixes SLE build
* Fri Sep  2 2011 saschpe@suse.de
- Update to version 0.15:
  * For loop docs fix and pointer iteration.
  * Pure decorators now implemented.
  * fix bug #707: optimised dict iteration over non-trivial expressions fail...
  * optimise object.pop() for sets
  * Py2.4 fix: PySet_Pop() appeared in Py2.5
  * Py3.3 test fix
  * Support module level control flow and Entry-level error on uninitialized
- Spec file cleanup:
  - Fixed wrong EOL encodings and non-excutable scripts
  - Set license to Apache-2.0
  - Run testsuite
* Wed Apr 20 2011 prusnak@opensuse.org
- updated to 0.14.1
  - changes too numerous to list, see the following for more details:
  * http://wiki.cython.org/ReleaseNotes-0.13
  * http://wiki.cython.org/ReleaseNotes-0.14
  * http://wiki.cython.org/ReleaseNotes-0.14.1
* Sun Jun  6 2010 dimstar@opensuse.org
- Use renewed python-macros, also for compatibility with other
  build targets.
* Sun Jun  6 2010 dimstar@opensuse.org
- Initial package of Cython 0.12.1
